/*
** (C) Copyright 2000 Gary King and Langche Zeng, All Rights Reserved.
** King@Harvard.Edu, lzeng@gwu.edu
** Current version available at http://GKing.Harvard.Edu
**
** CoxCCQ: Quantities of Interest from the Cox model with nested/density 
** case control data.
**
** This procedure computes risks/rates, risk/rate ratios and differences,
** and number needed to treat along with measures of uncertainty from
** Coxcc() model coefficient estimates, with corrections for risk set
** sampling (as in nested/density case control data). It implements
** suggestions in Gary King and Langche Zeng: "Inference in Case-Control
** Studies with Limited Auxiliary Information", available at
** http://GKing.Harvard.Edu.
**
**
** USAGE
**
** library maxlik;
** {b,vc,llik,ret} = Coxcc(t,d,x);  @ estimates coxcc parameters @
** {Q,U} = CoxccQ(x0,b,vc);         @ quantities of interest & uncertainties @
**
** INPUTS to coxccQ
**
** x0 = 2xk matrix of values of explanatory variables (without constant)
**      for ratios and differences of risks/rates and for number needed to
**      treat (to estimate what happens when x changes from x0[1,.] to
**      x0[2,.]); or mxk matrix of values of explanatory variables for
**      risks/rates.
**
** (b & vc are exactly as output from coxcc)
** b  = kx1 vector of coefficients;
** vc = kxk estimated variance matrix of b
**
** OUTPUTS
**
** Q = point estimate of risks/rates (mx1), or ratio or
**     difference of risks/rates or number needed to treat (1x1); 
** U = confidence intervals (1x2 for ratios, differences, and NNT or mx2
**     for risks/rates).
**
** GLOBALS
**
** _coxccQ_Q: define output Q Quantity of interest.
**              "P"   probability/risk,       Pr(y=1|x0)  (default)
**              "PR"  risk ratio,     Pr(y=1|x0[2,.]) / Pr(y=1|x0[1,.])
**              "PD"  risk difference, Pr(y=1|x0[2,.]) - Pr(y=1|x0[1,.])
**              "NNT" number needed to treat, 1/[Pr(y=1|x0[1,.]) - C]
**                    where C=min(Pr(y=1|x0[1,.]),Pr(y=1|x0[2,.])) 
**              "R"   rate (cumulative if rows(_coxccQ_rd)>1. same below)
**              "RR"  rate ratio
**              "RD"  rate difference
**                 
** _coxccQ_ci:   ci*100% confidence interval    (default=0.90)
**
** _coxccQ_bsims: 0 draw simulations via normal approximation (default);
**                _coxccQ_sims by rows(b) matrix of simulated b's. 
**                Useful for saving time in Monte Carlo experiments.  
**  
** _coxccQ_sims: number of simulations. reduce for speed, increase for 
**               precision  (default=10000)
**
** _coxccQ_alpha: with probability alpha, the risk set sampling fractions
**                are believed to be in their prior bounds [lo,hi] with 
**                unknown density; and with probability (1-alpha)/2 each 
**                in [0,lo] and [hi,1] with Jeffrey's invariance prior
**                density. (where "lo" and "hi" are vectors). default=.95
**
** _coxccq_r: specify risk sets information: failure times spanning the
**            interval on which QofI's are to be computed, and prior 
**            bounds for the corresponding risk set sampling fractions.
**              0 same interval as in data for coxcc, and no risk set 
**                sampling (default)
**              1 either use different time interval or correct for 
**                risk set sampling or both. _coxcc_rd must be set, which
**                has 3 columns. The first column gives the unique failure 
**                time(s) within the relevant interval, the next two columns 
**                give the prior lower and upper bounds for the corresponding
**                risk set sampling fraction(s).
**
** _coxccQ_pop: size of the population under study. lo=0 is recoded to 
**             1/_coxcccQ_pop. Default is 270000000 (US pop).             
**
*/

declare matrix _coxccQ_q="P";
declare matrix _coxccQ_ci=0.9;
declare matrix _coxccQ_sims=10000;
declare matrix _coxccQ_bsims=0;
declare matrix _coxccQ_alpha=.95;
declare matrix _coxccQ_r=0;
declare matrix _coxcc_rd={.};
declare matrix _coxccQ_pop=270000000;

proc (2)=coxccQ(x0,b,vc);
  local rset,nc,startr,endr,lo,hi,finfo,bsims,njeff,nbounds,r,r0,sumr,tau,
  hlo,hhi,i,Slo,Shi,s,phi,plo,q,qs,ss,c,qlo,qhi,cs,rs,ci,rlow,rhi,interval;

  /* check inputs */
  _coxccQ_q=upper(_coxccQ_q);
  if in(_coxccQ_q,"PR"|"PD"|"NNT"|"RR"|"RD",0);
    if rows(x0)/=2;
      "rows(x0) must be 2 with _coxccQ_q=PR, PD, NNT, RR, or RD.";end;
    endif;
  elseif _coxccQ_q$/="P" and _coxccQ_q$/="R";
    "_coxccQ_q must be P,PR,PD,R,RR,RD, or NNT. (use quote marks)";end;
  endif;
  if rows(_coxccQ_sims)/=1 or _coxccQ_sims<1;
    "_coxcc_sims error";end;
  endif;    
  if not(scalzero(_coxccQ_bsims));
    if rows(_coxccQ_bsims)/=_coxccQ_sims or cols(_coxccQ_bsims)/=rows(vc);
      "_coxccQ_bsims error";end;
    endif;
  endif;
  if _coxccQ_r==1;
    if scalmiss(_coxccq_rd);
      "_coxccq_rd must be defined if _coxccq_r=1"; end;
    elseif in(_coxccQ_rd[.,1],_coxcc_r[.,1],1)==0;
    "error specifying failure time values."; end;
    elseif minc(minc(_coxccQ_rd[.,2:3]))<0 or maxc(maxc(_coxccQ_rd[.,2:3]))>1;
    "bounds for risk set sampling fractions must be within [0,1]";end;
    endif;
  endif;
    
   /* simulations of beta */
  if scalzero(_coxccQ_bsims);          @ normal approx @
    bsims=rndmn(b,vc,_coxccQ_sims);    @ _coxccq_sims by k @
  else;
    bsims=_coxccQ_bsims;               @ input sims @
  endif;
  
  r0=exp(x0*bsims');                   @ rows(x0) by sims @ 
  if _coxccq_q$=="RR";                 @ simple! @ 
     s=r0[2,.]./r0[1,.];
     s=s';
     q=median(s);                      
     gosub uncer;
     retp(q,interval);
     end;
  endif;
   
  /* Otherwise need a lot more stuff for hazard rates */
  
  /* info on risk sets used */
  if _coxccQ_r==0;
    _coxccq_rd=_coxcc_r[.,1]~ones(rows(_coxcc_r),2);
  endif;
  _coxccQ_rd=sortc(_coxccQ_rd,1);        @ sort by unique failure times @
  lo=_coxccQ_rd[.,2];                   @ lower bounds of Tj @
  lo=recode(lo,lo.==0,1/_coxccq_pop);         
  hi=_coxccQ_rd[.,3];                   @ upper bounds of Tj @
  finfo=(lo==hi);                      @ Tj's known exactly @
  rset=selif(_coxcc_r,_coxcc_r[.,1].>=minc(_coxccQ_rd[.,1]) .and
        _coxcc_r[.,1].<=maxc(_coxccQ_rd[.,1]));
  nc=rset[.,2];        @ number of diseases in each of the used risk sets @
  startr=rset[.,3];    @ starting obs number of the risk set @
  endr=rset[.,4];      @ ending obs number of the risk set @

  /* Jeffrey's invariate prior for the area outside of the manski bounds */
  njeff=round(_coxccq_sims*(1-_coxccq_alpha)/2);
         @1/2*number of Jeffrey's rnd numbers to draw for tau @
  nbounds=round(_coxccq_sims*_coxccq_alpha); @ sims of lo/hi bounds @
  if njeff>0;
    tau=rndu(njeff,1).*lo';              @ njeff by rows(lo) @
    tau=tau|(rndu(njeff,1).*(1-hi')+hi'); 
    tau=(sin((pi/2)*tau))^2;           @ sims*(1-alpha) by rows(lo),Jeffrey's @
  endif;
    
  /* more risk set info */
  r=exp(_coxcc_x*bsims');			@ rows(x) by sims @
  sumr=zeros(rows(nc),_coxccq_sims);            @ rows(nc) by sims @
  for i(1,rows(sumr),1);
    sumr[i,.]=sumc(r[startr[i]:endr[i],.])';      
  endfor;
  sumr=1./sumr;
  
  /* (cumulative) hazard rate */
  hlo=sumr[.,1:nbounds].*lo;
  if njeff>0;
    hlo=hlo~(sumr[.,(nbounds+1):cols(sumr)].*tau');
  endif;
  qlo=sumc(hlo.*nc);             @ simsx1 cumulative base hazard rate, lo@
  hlo=r0'.*qlo;                  @ simsxrows(x0) cumulative hazard rate, lo@ 

  if not(finfo);
    hhi=sumr[.,1:nbounds].*hi;
    if njeff>0;
      hhi=hhi~(sumr[.,(nbounds+1):cols(sumr)].*tau');
    endif;
    qhi=sumc(hhi.*nc);             @ simsx1 cumulative base hazard rate, hi@
    hhi=r0'.*qhi;                  @ simsxrows(x0) cumulative hazard rate,hi@
  endif;    

  
  /* Quantities  of interest */
  if _coxccq_q$=="P";		              @ probability @
    slo=1-exp(-hlo);
    if finfo;	                              @ full info on Tj's @
      s=slo;
      q=median(s);                            @ rows(x0)x1 @
      gosub uncer;
    else;
      shi=1-exp(-hhi);  
      s=slo~shi;
      gosub uncer;
      q=meanc(interval');
    endif;
     
  elseif _coxccq_q$=="PR";		        @ risk ratio @
    slo=(1-exp(-hlo[.,2]))./(1-exp(-hlo[.,1])); 
    if finfo;	                                @ full info on Tj's @
      s=slo;
      q=median(s);                              @ 1x1 @
      gosub uncer;
    else;
      shi=(1-exp(-hhi[.,2]))./(1-exp(-hhi[.,1]));
      s=minc((Slo~Shi)')~maxc((Slo~Shi)'); 
      gosub uncer;
      q=meanc(interval');
    endif;
     
  elseif _coxccq_q$=="R";                      @ (cumulative) rate @
    slo=hlo;
    if finfo;	                                @ full info on Tj's @
      s=slo;
      q=median(s);                              @ rows(x0)x1 @
      gosub uncer;
    else;
      shi=hhi;
      s=slo~shi;
      gosub uncer;
      q=meanc(interval');
    endif;
   
  elseif _coxccq_q$=="RD";                      @ (cum.) rate difference @
    slo=hlo[.,2]-hlo[.,1];
    if finfo;	                                 @ full info on Tj's @
      s=slo;
      q=median(s);                               @ 1x1 @
      gosub uncer;
    else;
      shi=hhi[.,2]-hhi[.,1];
      s=minc((Slo~Shi)')~maxc((Slo~Shi)');
      gosub uncer;
      q=meanc(interval');
    endif;    
     
  else;                                           @ Risk difference or NNT @
    plo=1-exp(-hlo);                              @ Pr at lo of Tj @
    if _coxccq_q$=="PD";                          @ Risk diff. @
      slo=plo[.,2]-plo[.,1];
    else;                                         @ NNT @
      slo=1./(plo[.,1]-minc(plo[.,1]'|plo[.,2]'));
    endif;
    if finfo;	                                  @ full info on Tj's @
      s=slo;
      q=median(s);                                @ 1x1 @
      gosub uncer;
    else;                                         @ lo/hi bounds @
      phi=1-exp(-hhi);                              @ Pr at hi of Tj @
      shi=phi[.,2]-phi[.,1];
      r0=r0';                                     @ sims by rows(x0)@
      qs=(1./(r0[.,2]-r0[.,1])).*ln(r0[.,2]./r0[.,1]);  @ simsx1, Q* @
      ss=exp(-r0[.,1].*qs)-exp(-r0[.,2].*qs);     @ PD at Q* @
      c=((qlo.<=qs).and(qs.<=qhi));               @ [qlo,qhi] contains Q*? @
      s=minc((Slo~Shi~ss)')~maxc((Slo~Shi~ss)');  @ if yes @
      s=c.*s+(1-c).*(minc((Slo~Shi)')~maxc((Slo~Shi)'));      
      s=substute(s,(s[.,1].<-1)~(s[.,2].>1),-1~1); @ s must be in [-1,1]@
      if _coxccq_q$=="NNT"; @deriving bounds for NNT based on that for RD @
        s=1./(-substute(s,s.>=0,0));
        s=minc(s')~maxc(s');
      endif;
      gosub uncer;
      q=meanc(interval');
    endif;

  endif;
    
  retp(q,interval);
  
  /* uncertainty. */
  uncer:  
  cs=cols(s);
  rs=rows(s);
  for i (1,cs,1);
    s[.,i]=sortc(s[.,i],1);
  endfor;
   
  ci=(1-_coxccq_ci)/2; 
  rhi=int((1-ci)*rs);		@ row of s to take for the upper bound of CI@
  rlow=int(ci*rs);		@ row of s to take for the lower bound of CI@
  if rlow<1;
     rlow=1;
  endif;

  if _coxccq_q$=="RR";
    interval=(s[rlow,.]')~(s[rhi,.]');
  elseif finfo;                     @ no lo/hi bounds @
    interval=(s[rlow,.]')~(s[rhi,.]');
  else;                                  @ s is lo~hi @
    interval=(s[rlow,1:cs/2]')~(s[rhi,(cs/2+1):cs]');
  endif;

  return;

endp;






